'
' Licensed Materials - Property of IBM
' 5748-XX8
' (c) Copyright International Business Machines Corp. 2006
' All Rights Reserved
' US Government Users Restricted Rights - Use, duplication or disclosure
' restricted by GSA ADP Schedule Contract with IBM Corp.
'
Option Explicit
'
' Script to configure the FIM Plug-in into the IIS web server.
' After any config / unconfig actions, IIS will have to be 
' restarted in order to pick up the changes.
'
' Usage: 
' 
' fimpiiiscfg -action config [-webserver <WMI WebServer ID>]*
' fimpiiiscfg -action unconfig
' fimpiiiscfg -action list-webservers
'
Public Const FIMPI_DESCRIPTION     = "IBM FIM Plug-in"  ' The pretty description of our plug-in
Public Const FIMPI_FILTER_NAME     = "IBMFIMPI"         ' The name of the filter
Public Const FIMPI_EXTENSION_GROUP = "IBMFIMPI"         ' The extension group our plug-in is defined as belonging to
Public Const FIMPI_SPS_DIRECTORY   = "SPS"              ' The directory to claim as our proxy directory.

' The main line
Dim locatorObj, providerObj
Set locatorObj  = CreateObject("WbemScripting.SWbemLocator")
Set providerObj = locatorObj.ConnectServer("localhost", "root/MicrosoftIISv2")

dim action
action = getAction()

if action = "config" then
        configure providerObj
elseif action = "unconfig" then
        unconfigure providerObj
elseif action = "list-webservers" then
        printAvailableServers providerObj
else
        WScript.echo "Invalid action (" & action & ") specified."
        printUsage
end if

' Prints out a usage message
sub printUsage
        WScript.echo "fimpiiiscfg.vbs : Configure / unconfigure the FIM Plug-in into the IIS web server"
        WScript.echo "fimpiiiscfg.vbs -action config [-webserver <WMI WebServer ID>]*"
        WScript.echo "fimpiiiscfg.vbs -action unconfig"
        WScript.echo "fimpiiiscfg.vbs -action list-webservers"
end sub

' Retrieves the action the user specified on the command line
function getAction
        if WScript.Arguments.Count < 2 then
                getAction = ""
        elseif not WScript.Arguments.Item(0) = "-action" then
                getAction = ""
        else
                getAction = WScript.Arguments.Item(1)
        end if
end function

'
' Configures the FIM Plug-in for IIS
'
' \param providerObj : The WMI provider we're connected to
'
sub configure (providerObj)
        dim webserver, webserversArray
        dim filePath, pluginPath
        
        filePath   = getFilePath()
        pluginPath = filePath & "\fimpi-iis6.dll"

        webserversArray = getWebserversToConfigure(providerObj)

        if isnull(webserversArray) then
                printUsage
        else
                for each webserver in webserversArray 
                        createSPSVirtualDirectory providerObj, webserver, filePath, pluginPath
                        addFIMPIFilter providerObj, webserver, pluginPath
                next
                addFIMPIExtensionToList providerObj, pluginPath
        end if

end sub

function getFilePath()
        dim objShell
        set objShell = WScript.CreateObject("WScript.Shell")
        
        dim basePath
	basePath = objShell.RegRead("HKLM\Software\IBM\Tivoli Federated Identity Manager\6.1.1.0\WebServerPlugin\BasePath")

        getFilePath = basePath & "web"
end function
'
' Get the webservers to configure from the command line
'
' \param providerObj : The WMI provider we're connected to
'
function getWebserversToConfigure (providerObj) 
        dim argIdx
        dim webserverCount
        webserverCount = 0
                
        for argIdx = 0 to WScript.Arguments.Count - 2 step 2
                if WScript.Arguments.Item(argIdx) = "-webserver" then
                        webserverCount = webserverCount + 1
                end if
        next
        
        if webserverCount > 0 then
                dim webserversArray()
                redim webserversArray(webserverCount - 1)
                dim webserverIdx
                webserverIdx = 0
                
                for argIdx = 0 to WScript.Arguments.Count - 2 step 2
                        if WScript.Arguments.Item(argIdx) = "-webserver" then
                                webserversArray(webserverIdx) = WScript.Arguments.Item(argIdx + 1)
                                webserverIdx = webserverIdx + 1
                        end if
                next
                
                getWebserversToConfigure = webserversArray
        else
                getWebserversToConfigure = null
        end if
        
end function

'
' Unconfigure the FIM Plug-in from the IIS web server
'
' \param providerObj : The WMI provider that we're connected to
'
sub unconfigure (providerObj)

        dim webserverObj

        for each webserverObj in providerObj.execQuery("select * from IIsWebServer") 
                ' Remove the filter and SPS directory from all web servers
                removeFIMPIFilter          providerObj,  webserverObj.name
                removeSPSVirtualDirectory  providerObj,  webserverObj.name
        next
        
        ' Remove the FIM Plug-in from the extension list
        removeFIMPIExtensionFromList providerObj
        
end sub

'
' Print the list of available web servers. The list contains the WMI name and
' the description, with a single space character between the two.
'
' \param providerObj : The WMI provider that we're connected to
'
sub printAvailableServers (providerObj)

        Dim webservers, server
        Set webservers = providerObj.execQuery("select * from IIsWebServer")

        for each server in webservers 
                Dim serverSettings
                set serverSettings = providerObj.Get("IIsWebServerSetting='" & server.name & "'")
                WScript.echo server.name & " " & serverSettings.ServerComment
        next

end sub

'
' This function creates the SPS virtual directory and installs the FIM Plug-in 
' extension as the wildcard script map for this virtual directory.
'
' \param providerObj   : The WMI provider object that we're connected to
' \param webserverName : The WMI name of the webserver to perform this task
'                        on, eg. "W3SVC/1"
' \param filePath      : The full path of the directory to base this virtual
'                        directory on.
' \param pluginPath    : The full path of the FIM Plug-in
'
sub createSPSVirtualDirectory (providerObj, webserverName, filePath, pluginPath)
        
        Dim vdirClassObj, scriptMapClassObj
        Dim vdirObj, scriptMapObj
        dim vdirName
        vdirName = webserverName & "/root/" & FIMPI_SPS_DIRECTORY
        
        ' Create a new virtual directory instance
        Set vdirClassObj = providerObj.Get("IIsWebVirtualDirSetting")
        Set vdirObj      = vdirClassObj.SpawnInstance_()

        ' Configure the virtual directory instance
        vdirObj.Name = vdirName
        vdirObj.Path = filePath
        vdirObj.AccessFlags = 5 ' Read, Execute

        ' Create a new script map instance
        Set scriptMapClassObj = providerObj.Get("ScriptMap")
        Set scriptMapObj      = scriptMapClassObj.SpawnInstance_()
                
        ' Set our extension as a default script map
        scriptMapObj.Extensions = "*"
        scriptMapObj.Flags = 3
        scriptMapObj.IncludedVerbs = "All"
        scriptMapObj.ScriptProcessor = pluginPath
                
        ' Need COM Array format
        vdirObj.ScriptMaps = Array(scriptMapObj)
                
        'Save the virtual dir
        vdirObj.Put_()
                
        ' Create a pooled application on the new virtual directory. 
        Set vdirObj = providerObj.Get("IIsWebVirtualDir='" & vdirName & "'") 
        vdirObj.AppCreate2(2) 

        ' Set the application name. 
        Set vdirObj = providerObj.Get("IIsWebVirtualDirSetting='" & vdirName & "'") 
        vdirObj.AppFriendlyName = FIMPI_DESCRIPTION
        vdirObj.Put_()

end sub

'
' This function removes the SPS virtual directory that we have created from 
' the specified web server. It checks that the description matches our
' description before deleting the object, so hopefully we won't clobber any
' innocent bystanding SPS directories.
'
' \param providerObj   : The WMI provider object that we're connected to
' \param webserverName : The WMI name of the webserver to perform this task
'                        on, eg. "W3SVC/1"
'
sub removeSPSVirtualDirectory (providerObj, webserverName)
        dim vdirObj, vdirName
        vdirName = webserverName & "/root/" & FIMPI_SPS_DIRECTORY
        
        ' Disable automatic error checking so that we can safely try to access
        ' something that may not exist
        on error resume next
        
        ' Check that this virtual directory exists and belongs to us
        set vdirObj = providerObj.Get("IIsWebVirtualDirSetting='" & vdirName & "'")
        
        ' Check whether we actually succeeded in getting an object
        if Err.number = 0 then
                ' Check that the virtual directory is ours
                if vdirObj.AppFriendlyName = FIMPI_DESCRIPTION then
                        providerObj.Delete("IIsWebVirtualDirSetting='" & vdirName & "'")
                        providerObj.Delete("IIsWebVirtualDir='" & vdirName & "'")
                end if
        end if
        
        on error goto 0
        
end sub

'
' This function adds the FIM Plug-in filter to the filter list for the web 
' server specified
' 
' \param providerObj   : The WMI provider that we're connected to
' \param webserverName : The WMI name of the web server into which
'                        the filter will be installed, eg. W3SVC/1
' \param pluginPath    : Full path to the FIM Plug-in
'
sub addFIMPIFilter (providerObj, webserverName, pluginPath)

        dim filterClassObj, filterObj
        dim filterName

        filterName = webserverName & "/Filters/" & FIMPI_FILTER_NAME
        
        ' Create our filter
        set filterClassObj = providerObj.Get("IIsFilterSetting")
        set filterObj      = filterClassObj.SpawnInstance_()
        
        filterObj.Name = webserverName & "/Filters/" & FIMPI_FILTER_NAME
        filterObj.FilterPath = pluginPath
        filterObj.NotifyOrderHigh = true
        filterObj.FilterDescription = FIMPI_DESCRIPTION
        
        ' Save it to the metabase
        filterObj.Put_()
        
        dim filtersObj, filterList

        ' Get the configured filters for the web server 
        set filtersObj = providerObj.Get("IIsFiltersSetting='" & webserverName & "/Filters'")
        filterList = filtersObj.FilterLoadOrder

        ' Prepend our filter to the list of filters to be loaded
        if not filterList = "" then
                filterList = FIMPI_FILTER_NAME & "," & filterList
        else
                filterList = FIMPI_FILTER_NAME
        end if
        filtersObj.FilterLoadOrder = filterList
        
        ' Save the changes
        filtersObj.Put_()
        
end sub

'
' This function removes the FIMPI Filter from the webserver's load order and 
' then deletes the FIMPI Filter object from the WMI objectspace.
'
' \param providerObj   : WMI provider we're connected to
' \param webserverName : The WMI name of the web server into which
'                        the filter will be installed, eg. W3SVC/1
'
sub removeFIMPIFilter (providerObj, webserverName)
        
        dim filtersObj, filtersList
                

        ' Disable automatic error checking so that we can safely try to access
        ' something that may not exist
        on error resume next
        set filtersObj = providerObj.Get("IIsFiltersSetting='" & webserverName & "/Filters'")

        ' Check if it worked    
        if Err.number = 0 then
                on error goto 0

                filtersList = filtersObj.FilterLoadOrder
        
                ' Split it info filters
                dim filtersArray, index, fimpiFilterIndex
                filtersArray = Split(filtersList, ",", -1, 1)
                fimpiFilterIndex = -1
                for index = 0 to UBound(filtersArray)
                        if filtersArray(index) = FIMPI_FILTER_NAME  then
                                fimpiFilterIndex = index
                                index = UBound(filtersArray) + 1
                        end if
                next
                
                dim newFiltersList
                newFiltersList = ""
        
                ' Create a new list without our filter
                for index = 0 to UBound(filtersArray)
                        if not index = fimpiFilterIndex then
                                if index > 0 then
                                        newFiltersList = newFiltersList & ","
                                end if
                                newFiltersList = newFiltersList & filtersArray(index)
                        end if
                next
                
                ' Apply and save the change
                filtersObj.FilterLoadOrder = newFiltersList
                filtersObj.Put_()

                ' Remove the FIMPI Filter Object
                on error resume next    
                providerObj.Delete(webserverName & "/Filters/" & FIMPI_FILTER_NAME)
                on error goto 0

        end if  

end sub

'
' This function adds the FIMPI Extension to the Web Services Extension list.
'
' \param providerObj : The WMI provider that we're connected to
' \param pluginPath  : Full path to the FIM Plug-in
'
sub addFIMPIExtensionToList (providerObj, pluginPath)
        dim webserviceSettingObj
        dim webSvcExtRestrictionListClass, websvcExtRestrictionListObj
        
        ' Get the webservice setting object
        set webserviceSettingObj = providerObj.Get("IIsWebServiceSetting='W3SVC'")
        
        ' Create a web service extension restriction object
        set webSvcExtRestrictionListClass = providerObj.Get("WebSvcExtRestrictionList")
        set webSvcExtRestrictionListObj = webSvcExtRestrictionListClass.SpawnInstance_()

        ' Configure the IBM FIM Plug-in's extension restriction 
        webSvcExtRestrictionListObj.Access = "1"                         ' Access Allowed
        webSvcExtRestrictionListObj.Deletable = "1"                      ' Deletable
        webSvcExtRestrictionListObj.Description = FIMPI_DESCRIPTION
        webSvcExtRestrictionListObj.FilePath = pluginPath
        webSvcExtRestrictionListObj.ServerExtension = FIMPI_EXTENSION_GROUP

        ' Add the FIM Plug-in's extension restriction to the extension restriction list 
        dim elementArray
        elementArray = webserviceSettingObj.WebSvcExtRestrictionList
        redim preserve elementArray(UBound(elementArray) + 1)
        set elementArray(UBound(elementArray)) = webSvcExtRestrictionListObj
        webserviceSettingObj.WebSvcExtRestrictionList = elementArray
        
        ' Save the changes
        webserviceSettingObj.Put_()

end sub

'
' This function removes the FIMPI Extension from the Web Services Extension
' list.
'
' \param providerObj : The WMI provider object that we're connected to
'
sub removeFIMPIExtensionFromList (providerObj) 

        dim webserviceSettingObj
        
        ' Get the webservice setting object
        set webserviceSettingObj = providerObj.Get("IIsWebServiceSetting='W3SVC'")
        
        dim elementArray, fimpiIndex, index
        elementArray = webserviceSettingObj.WebSvcExtRestrictionList
        fimpiIndex = -1
        
        ' Find out which element is the FIM Plug-in
        for index = 0 to UBound(elementArray)
                if elementArray(index).Description = FIMPI_DESCRIPTION then
                        fimpiIndex = index
                        index = UBound(elementArray) + 1
                end if
        next

        ' If we found the FIM Plug-in, move it to the end of the list then decrease the list size
        if not fimpiIndex = -1 then
                dim tempObject
                set tempObject = elementArray(fimpiIndex)
                set elementArray(fimpiIndex) = elementArray(UBound(elementArray))
                set elementArray(UBound(elementArray)) = tempObject
                redim preserve elementArray(UBound(elementArray) - 1)
                webserviceSettingObj.WebSvcExtRestrictionList = elementArray
        end if

        ' Save the changes
        webserviceSettingObj.Put_()     

end sub

